#ifndef GUISCROLLWINDOW_H_
#define GUISCROLLWINDOW_H_

#include "GuiEffectWindow.h"
#include "GuiScrollBar.h"
#include "GuiImage.h"

#define FRAME_ID_SCROLL_MOVE	1000
#define FRAME_ID_SCROLL_STILL	1001
#define RES_SCROLL_UP			1002
#define RES_SCROLL_DOWN			1003
#define RES_SCROLL_BAR			1004
#define RES_SCROLL_IND_UP		1005
#define RES_SCROLL_IND_DN		1006

#define SCROLL_STEP				90
#define SCROLL_STEP_ANIM		20
#define DRAG_THRESHOLD			20

/**
 * \brief Implements a scrollable GUI window
 *
 * Given class implements a scrollable window. The scrollable window is built
 * using a window frame with a virtual size greater than the visible frame area
 * displayed in the window. The dimensions of the viewable area remain
 * constant, while the virtual frame size may vary. User may move the viewing
 * point across the virtual frame surface - such approach increases the
 * scrolling speed with a penalty of a greater memory usage.
 *
 * Scrollable window assumes that the window has a frame with a constant
 * identifier and all primitives that need to be scrolled must be created
 * inside that frame. Scrolling may be performed using on-screen buttons that
 * should be instantiated in a separate non-scrollable frame. Alternatively,
 * scrolling may be performed with mouse/touchscreen dragging.
 *
 * Scrollable window also encapsulates a scrollbar. If the scrollbar feature
 * is enabled, the scrollbar is automatically recalculated and redrawn whenever
 * the window is scrolled.
 *
 * \author Andrei Birjukov, Anti Sullin
 */

class GuiScrollWindow : public GuiEffectWindow
{
public:

	GuiScrollWindow(int idResource);
	virtual ~GuiScrollWindow();

	void enableDragScroll(bool flag);
	void enableButtonScroll(bool flag);

	void setDragThreshold(int threshold);
	void setScrollStep(int totalStep, int animStep);

	virtual void scrollUp();
	virtual void scrollDown();

	bool showWindow(bool state);

protected:

	void updateScrollData(bool doRedraw);
	virtual void eventHandler(s_event_t *event);
	virtual void notifyTouch(int notify, TouchItem item);

	/** The X coordinate of the point where touch occurred */
	int				pointX;

	/** The Y coordinate of the point where touch occurred */
	int				pointY;

	/** The width of the virtual frame size */
	int				frameVirtW;

	/** The height of the virtual frame size */
	int				frameVirtH;

	/** Actual (visible) width of the frame */
	int				frameW;

	/** Actual (visible) height of the frame */
	int				frameH;

	/** Step size of each scrolling operation */
	int				scrollStep;

	/** Animation size for the smooth window frame moving */
	int				animationStep;

	/** Distance that the pointer must move before the scrolling is executed */
	int				dragThreshold;

	/** Flag that shows if the scrolling is initiated or not */
	bool			isScrollInitialized;

	/** Whether the scrolling via touchscreen dragging event is enabled or not */
	bool			dragScroll;

	/** Whether the scrolling by onscreen buttons are enabled or not */
	bool			buttonScroll;

	/** Pointer to the frame that contains the scrollable content */
	GuiFrame*		scrollFrame;

	/** Pointer to the frame that contains the scrollbar components */
	GuiFrame*		stillFrame;

	/** Pointer to the scrollbar GUI object */
	GuiScrollBar*	scrollBar;

	/** Pointer to the up arrow indicator GUI object */
	GuiImage*		scrollIndUp;

	/** Pointer to the down arrow indicator GUI object */
	GuiImage*		scrollIndDn;

	/** Flag that shows whether the scrolling by dragging is currently happening or not */
	bool			isDragging;
};

#endif /*GUISCROLLWINDOW_H_*/
