/*
 * NetworkConnection.h
 *
 *  Created on: Nov 19, 2008
 *      Author: andrei
 */

#ifndef NETWORKCONNECTION_H_
#define NETWORKCONNECTION_H_

#include <string>
#include <list>

#include "Thread.h"
#include "PowerManager.h"
#include "NetworkEventListener.h"
#include "EventBroadcaster.h"

/** Information about a network connection. The structure is
 * commonly used to store both GPRS and WLAN network information.
 */
struct GENERIC_NETWORK_INFO
{
	/** Specifies if the network connection is active */
	bool			isConnected;
	/** Contains the name of the network, i.e. SSID in case of WLAN
	 * and operator name in case of GPRS*/
	std::string		networkName;
	/** Contains the network channel currently in use */
	int				networkChannel;
	/** Specifies the received signal strength in percent 0..100 */
	int				signalStrength;
	/** Specifies the acquired network IP address */
	std::string		networkAddress;
	/** Specified the hardware address of the network adapter (MAC) */
	std::string		hardwareAddress;
};

/**
 * \brief Network connection base class
 *
 * Implements the basic functionality of a network connection.
 * Encapsulates a state machine and a worker thread that maintains the network
 * connection and attempts to repair it when a failure is detected.
 *
 * This is an abstract class, it may not be instantiated and used directly.
 * Instead, a specific implementation should inherit from this class and implement
 * the declared netXXX methods.
 *
 * NetworkConnection is a power management aware class, it suspends the connection
 * when a power saving mode of the data terminal is entered and restores the
 * connection when returning from a power saving mode.
 */
class NetworkConnection : protected PowerEventListener,
	protected virtual Thread, public EventBroadcaster<NetworkEventListener>
{
public:

	NetworkConnection(int netType);
	virtual ~NetworkConnection();

	bool initialize();
	bool shutdown();

	bool connect();
	bool disconnect();

	void addListener(NetworkEventListener *pListener);

	/**
	 * \brief Get last error
	 * \return Returns the text message associated with a last error occured, NULL if none.
	 */
	const char* getLastError() { return lastError; };

	/**
	 * \brief Get network type
	 * \return Returns the network type identifier specified in the class constructor.
	 */
	int getNetworkType() { return networkType; }

	/**
	 * \brief Check connection state
	 * The implementation must implement this method to return a valid value.
	 * \return Returns true if the network connection is still available.
	 */
	virtual bool isConnectionUp() = 0;

	/**
	 * \brief Get network information
	 * \param netInfo Address to the structure where to store the retrieved network information
	 * The implementation must implement this method to fill the information structure.
	 * \return Returns true if the information was read successfully.
	 */
	virtual bool getNetworkInfo(GENERIC_NETWORK_INFO &netInfo) = 0;

	/**
	 * \brief Get network status
	 * \return Returns current network status.
	 */
	const char* getLastStatus() { return lastStatus; };

	/**
	 * \brief Get last network status or error message
	 * \return Returns last network status or error message.
	 */
	const char* getLastEventMsg() { return lastEventMsg; };

	NetworkEvent getLastEvent();

protected:

	/**
	 * \brief Network initialization routine
	 * The implementation must implement this method to initialize the network connection.
	 * \return Returns true if the network connection was initialized successfully.
	 */
	virtual bool netInitialize() = 0;

	/**
	 * \brief Network connection establish routine
	 * The implementation must implement this method to connect to a network.
	 * \return Returns true if network connection was established successfully.
	 */
	virtual bool netConnect() = 0;

	/**
	 * \brief Network disconnect routine
	 * The implementation must implement this method to disconnect from a network.
	 * \return Returns true if the network was disconnected successfully.
	 */
	virtual bool netDisconnect() = 0;

	/**
	 * \brief Network shutdown routine
	 * The implementation must implement this method to shutdown the network connection.
	 * \return Returns true if the network connection was shut down successfully.
	 */
	virtual bool netShutdown() = 0;

	virtual bool netRepair();
	virtual bool netSuspend();
	virtual bool netResume();

	virtual void run();

	virtual bool powerStateChange(PowerState oldState, PowerState newState);

	void eventNotify(NetworkEvent e);
	static bool getInterfaceAddresses(const char* ifName, char *ipAddr, char *hwAddr);

	void reportError(const char *error);
	void setStatus(const char *status);

private:

	/** Identifies the current state of the network connection. */
	enum NetworkConnectionState
	{
		/** Offline, not connected */
		OFFLINE,
		/** Repairing a dropped connection */
		REPAIR,
		/** Standby, not connected */
		STANDBY,
		/** Suspending, preparing the network connection for standby */
		SUSPEND,
		/** Resuming, restoring the network connection from standby */
		RESUME,
		/** Connected, fully operational */
		CONNECTED
	};

	void pollNetworkInfo();

	NetworkConnectionState	currentState;
	Mutex					stateMutex;
	int						cntPollInfo;
	int						cntNetRepair;

	NetworkEvent			lastEvent;
	const int				networkType;

	const char*				lastError;
	const char*				lastStatus;

	const char*				lastEventMsg;

	GENERIC_NETWORK_INFO	lastNetInfo;
};

/** \file */
#endif /* NETWORKCONNECTION_H_ */
