#ifndef POWERMANAGER_H_
#define POWERMANAGER_H_

/** Maximum time spent in power-save mode in loop */
#define POWER_SAVE_SLEEP	60		// maximum time spent in power-save mode in loop

#include <list>

#include "Thread.h"

/** Available power states of the mobile terminal. */
enum PowerState
{
	/** Device is fully operational, display is at maximum brightness */
	FULLY_OPERATIONAL = 0,
	/** Device is operational with full functionality,
	 * display is dimmed to the half brightness */
	DISPLAY_DIMMED,
	/** Device is operational with full functionality,
	 * display is at minimum brightness */
	DISPLAY_MINIMAL,
	/** Device is operational with full functionality,
	 * display is in power saving mode */
	DISPLAY_DISABLED,
	/** Device is awaken from power saving mode and runs in
	 * a temporary state with limited functionality */
	LOW_POWER_AWAKE,
	/** Device is in power saving mode, running on slow clock and
	 * awaiting wake-up interrupts */
	LOW_POWER_SLEEP,
	/** Device is powered down */
	POWER_DOWN
};

/** Number of power states defined */
#define NUM_POWER_STATES 7

/**
 * \brief Power event listener interface
 *
 * Interface for delivery of power manager events. Any class that wishes to listen
 * to power manager events should implement this interface.
 */
class PowerEventListener
{
public:
	/**
	 * \brief Power manager state change
	 *
	 * Notifies the listener class about a change in the power manager state.
	 * The method is called before the power manager executes all steps
	 * necessary to proceed to the specified new power state. The listener may
	 * reject current state transition by returning false. If the listener does
	 * not want to interfere, or allows transition to the new state, it should
	 * return true.
	 *
	 * \param oldState Specifies the old state of the power manager.
	 * \param newState Specifies the new state of the power manager.
	 */
	virtual bool powerStateChange(PowerState oldState, PowerState newState) = 0;
};

#include "EventBroadcaster.h"

/**
 * \brief Power manager class
 *
 * Power manager implements a singleton class that takes care of basic
 * power management functions of the mobile terminal. Power manager must
 * be initialized at the beginning of the application life and uninitialized
 * at the end. Power manager does not maintain any worker threads by itself.
 * Instead, an external supervisor or monitor should initiate power state
 * transitions.
 */
class PowerManager : public EventBroadcaster<PowerEventListener>
{
public:

	static bool initPowerManager();
	static bool deinitPowerManager();
	static PowerManager* getPowerManager();

	virtual ~PowerManager();

	void enableTransitions(bool flag);
	bool setPowerMode(PowerState newState);

	bool kickIdle();

	static int kickIdleProc();
	int getIdleDuration();

	void ledEnable(bool red, bool green);

	void keybSetBackLight(unsigned char level);
	void keybFadeBackLight(unsigned char level);

	bool oledEnable(bool flag);
	bool oledSetBrightness(unsigned char level);
	bool oledFadeBrightness(unsigned char level);
	bool oledFreeze(bool flag);
	void oledSetMaxBrightness(int maxLevel);

	void buzzerPlay(int freq, int duration);

	bool gprsPowerOn();
	bool gprsPowerOff();
	void gprsReset();
	bool gprsGetPower();
	void gprsSetDTR(bool active);

	bool btPowerOn();
	bool btPowerOff();

	void mpciEnableRadio(bool enable);
	void mpciEnable3V3(bool enable);

	bool scannerPowerUp();
	bool scannerPowerDown();
	bool scannerSetAim(bool flag);

	bool rtcGetAlarmState();
	bool rtcSetAlarm(int delay);
	bool rtcEnableAlarm(bool flag);
	bool rtcSetTime(time_t now);
	bool rtcGetTime(time_t *pNow);

	int getBatteryCapacity(int *pVoltage = NULL);
	int getBatteryVoltage();
	int getTempReading();
	int getAmbientLight();
	PowerState getCurrentState();

	bool isUsbConnected();

	void setPowerSaveSleep(int sec);

protected:

	PowerManager();

	bool initialize();
	bool deinitialize();

private:

	int findSpeakerDevice();

	Mutex			powerMutex;
	time_t			idleBegin;
	PowerState		currentState;

	int				buzzerDriver;
	int				powerDriver;
	int				rtcDriver;

	bool			enableStateSwitch;
	int				powerSaveSleep;

	Mutex			displayMutex;
	bool			displayEnabled;
	int				maxDisplayBrightness;
	int				batteryVoltage;

	static PowerManager* powerManager;
};

/** \file */

#endif /*POWERMANAGER_H_*/
