#ifndef SERIALPORT_H_
#define SERIALPORT_H_

#include <unistd.h>
#include <stdint.h>
#include <termios.h>

#include <string>

/**
 * \brief Serial port interface class
 *
 * Serial port interface class describes a common interface for
 * physical and Bluetooth serial ports.
 */
class SerialPortInterface {
public:
	/**
	 * \brief Reopen serial port
	 * Serial port implementation should override this function to implement port reopening method.
	 * It should be used to initialize the port to its last settings.
	 *
	 * \return Return true, if the port was reopened successfully.
	 */
	virtual bool reOpen() = 0;

	/**
	 * \brief Query port status
	 * Serial port implementation should override this function to implement port query method.
	 * This method can be used to check if the port is opened or not since serial port can be
	 * closed by the system using the HUP command.
	 *
	 * \return Returns true if the port is opened.
	 */
	virtual bool isPortOpen() = 0;

	/**
	 * \brief Read data from serial port
	 * Serial port implementation should override this function to implement the data reading method to
	 * receive specified number of bytes from the serial port.
	 *
	 * \param pData Points to the data bu7ffer.
	 * \param maxLen Specifies the number of bytes to be read.
	 *
	 * \return Returns number of bytes actually received from serial port or -1 on error.
	 */
	virtual ssize_t readData(void *pData, size_t maxLen) = 0;

	/**
	 * \brief Write data to serial port
	 * Serial port implementation should override this function to implement the method for writing
	 * data to serial port.
	 *
	 * \param pData Points to the data buffer.
	 * \param dataLen Specifies the number of bytes to write
	 *
	 * \return Returns the number of bytes actually written to the serial port.
	 */
	virtual ssize_t writeData(const void *pData, size_t dataLen) = 0;

	/**
	 * \brief Get number of pending bytes
	 * Serial port implementation should override this function to implement method for detecting
	 * the number of bytes currently pending in the receive buffer.
	 *
	 * The method should block until timeout expires or the data arrives to serial port. If there are
	 * bytes pending in the receive buffer already, the function should return immediately.
	 *
	 * \param timeOut Specifies timeout in milliseconds.
	 *
	 * \return Returns the number of bytes pending for read or 0 if no bytes are pending.
	 */
	virtual size_t numInputPending(int timeOut) = 0;

	/**
	 * \brief Check if there is input data
	 * Serial port implementation should override this function to implement method for detecting
	 * if there is input data waiting in the receive buffer.
	 *
	 * The method should block if there is no data currently pending for receive until the timeout
	 * expires or data arrives. If there are bytes pending in the receive buffer already, the function
	 * should return immediately.
	 *
	 * \param timeOut Specifies the timeout in milliseconds.
	 * \return Returns true if there is data pending for reading.
	 */
	virtual bool isInputPending(int timeOut) = 0;

	/**
	 * \brief Flush serial port input
	 * Serial port implementation should override this function to implement method for discarding all
	 * serial port input currently pending for receiving.
	 */
	virtual void flushInput() = 0;

	/**
	 * \brief Flush serial port output
	 * Serial port implementation should override this function to implement method for discarding all
	 * serial port output currently pending for transmit.
	 */
	virtual void flushOutput() = 0;

	/**
	 * \brief Drain serial port output
	 * Serial port implementation should override this function to implement method for waiting until all
	 * output data written to the serial port has been transmitted.
	 */
	virtual void drainOutput() = 0;
};

/** Serial port flow control mode */
enum S_FLOW_CONTROL
{
	/** Flow control is disabled */
	S_FLOW_NONE = 0,
	/** Sofware flow control (XON/XOFF) */
	S_FLOW_SOFT,
	/** Hardware flow control (RTS/CTS) */
	S_FLOW_HARD,
	/** Both hardware and software flow control */
	S_FLOW_BOTH
};

/** Serial port parity mode */
enum S_PARITY_CONTROL
{
	/** Parity check is disabled */
	S_PARITY_NONE = 0,
	/** Odd parity check */
	S_PARITY_ODD,
	/** Even parity check */
	S_PARITY_EVEN
};

// serial port configuration structure

/** Serial port mode attributes. */
struct SERIAL_PORT_MODE
{
	/** Serial port name */
	const char *portName;
	/** Serial port baud rate */
	unsigned long portSpeed;
	/** Number of serial port data bits per symbol */
	int charBits;
	/** Serial port parity mode */
	S_PARITY_CONTROL parityMode;
	/** Number of serial port stop bits per symbol */
	int stopBits;
	/** Serial port flow control mode */
	S_FLOW_CONTROL flowControl;
};

/**
 * \brief Serial port class
 *
 * Serial port class is a wrapper for generic serial port I/O.
 * The class is meant to simplify the handling of serial ports.
 */
class SerialPort: public SerialPortInterface
{
public:
	SerialPort();
	virtual ~SerialPort();

	bool open(const char* portDev);
	bool close();
	virtual bool reOpen();
	virtual bool isPortOpen();

	virtual ssize_t readData(void *pData, size_t maxLen);
	virtual ssize_t writeData(const void *pData, size_t dataLen);
	virtual size_t numInputPending(int timeOut);
	virtual bool isInputPending(int timeOut);

	virtual void flushInput();
	virtual void flushOutput();
	virtual void drainOutput();

	bool setSpeed(unsigned long speed);
	bool setFlowControl(S_FLOW_CONTROL flow);
	bool setStopBits(int bits);
	bool setCharBits(int bits);
	bool setParity(S_PARITY_CONTROL parity);
	void sendBreak();

	void setRTS(bool enable);
	void setDTR(bool enable);
	bool getCTS();

protected:

	void initConfig();

	/** Port device file handle */
	int devFile;
	/** Port device file name */
	std::string devName;

	/** If set to true, port is closed by the system using HUP */
	bool hup;

	/** If set to true, the stored serial port parameters are valid and do not need reconfiguration */
	bool lastAttrValid;
	/** Stores serial port parameters */
	struct termios lastAttr;
};

/** \file */
#endif /*SERIALPORT_H_*/
