/*
 * SystemMonitor.h
 *
 *  Created on: Apr 16, 2009
 *      Author: andrei
 */

#ifndef SYSTEMMONITOR_H_
#define SYSTEMMONITOR_H_

#include <exynthoo/GuiTimer.h>
#include <time.h>

/**
 * \brief System monitor class
 *
 * Implements system monitor functionality. System monitor periodically polls
 * system parameters such as the remaining battery capacity, amount of detected
 * ambient light, internal temperature, and calls registered event listeners.
 *
 * System monitor detects critical conditions and notifies registered listeners.
 * System monitor tracks the level of ambient lighting and enables or disables
 * keypad backlight when required.
 *
 * System monitors also tracks device idle time and automatically cycles
 * through power modes when the device remains idle.
 *
 */
class SystemMonitor : public GuiTimer
{
public:

	SystemMonitor();
	virtual ~SystemMonitor();

	static bool initialize();
	static void shutdown();
	static SystemMonitor* getMonitor();

	virtual void setPowerDelays(int dim, int min, int off, int save, int down);
	virtual void getPowerCounters(int *counters); //[NUM_POWER_STATES]
	virtual void setKeyboardBrightness(int b);
protected:

	/** Holds the instance of this class. */
	static SystemMonitor *monitor;

	virtual bool init();
	virtual void close();

	virtual void onTimer();

	virtual void handlePower();
	virtual void handleBattery();
	virtual void handleTemperature();
	virtual void handleLighting();

	/**
	 * Specifies the time delay after which the display is dimmed to half
	 * power if the device remains idle, in seconds.
	 */
	int displayDimDelay;
	/**
	 * Specifies the time delay after which the display is dimmed to
	 * lowest setting if the device remains idle, in seconds.
	 */
	int displayMinDelay;
	/**
	 * Specifies the time delay after which the display is disabled if
	 * the device remains idle, in seconds.
	 */
	int displayOffDelay;
	/**
	 * Specifies the time delay after which the system goes to low-power
	 * sleep if the device remains idle, in seconds.
	 */
	int powerSaveDelay;
	/**
	 * Specifies the time delay after which the system shuts down if the
	 * device remains idle, in seconds.
	 */
	int powerDownDelay;

	/**
	 * Specifies the time interval in which the remaining battery capacity
	 * is queried, in seconds. Registered listeners are also called every
	 * time.
	 */
	int batteryPollInterval;
	/**
	 * Specifies the time interval in which the internal temperature sensor
	 * is queried, in seconds. Registered listeners are also called every
	 * time.
	 */
	int tempPollInterval;
	/**
	 * Specifies the time interval in which the ambient lighting sensor
	 * is queried, in seconds. Registered listeners are also called every
	 * time.
	 */
	int lightPollInterval;

	/**
	 * Specifies the maximum keypad brightness, in relative units (0..255).
	 */
	int keypadMaxBrightness;

	/**
	 * Map of power state counters
	 */
	unsigned int powerStateCounter[NUM_POWER_STATES];

private:

	time_t lastBatteryPoll;
	time_t lastTempPoll;
	time_t lastLightPoll;
	time_t sleepStart;

	bool usbConnected;
};

#endif /* SYSTEMMONITOR_H_ */
